@@ -1,5 +1,5 @@
 /* BFD back-end for Intel 386 PECOFF files.
-   Copyright (C) 1995-2017 Free Software Foundation, Inc.
+   Copyright (C) 1995-2021 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -21,11 +21,14 @@
 #include "sysdep.h"
 #include "bfd.h"
 
-#define TARGET_SYM 		i386_pe_vec
-#define TARGET_NAME 		"pe-i386"
+#define TARGET_SYM		i386_pe_vec
+#define TARGET_NAME		"pe-i386"
+#define TARGET_SYM_BIG		i386_pe_big_vec
+#define TARGET_NAME_BIG		"pe-bigobj-i386"
 #define COFF_WITH_PE
-#define PCRELOFFSET 		TRUE
-#define TARGET_UNDERSCORE 	'_'
+#define COFF_WITH_PE_BIGOBJ
+#define PCRELOFFSET		TRUE
+#define TARGET_UNDERSCORE	'_'
 #define COFF_LONG_SECTION_NAMES
 #define COFF_SUPPORT_GNU_LINKONCE
 #define COFF_LONG_FILENAMES
